indexing

   description: "Class that covers POSIX memory mapping."

   author: "Berend de Boer"
   date: "$Date: Sat, 07 Oct 2000 14:31:08 +0200 $"
   revision: "$Revision: 1.1 $"

class 

   POSIX_MEMORY_MAP

inherit
   
   POSIX_BASE
   
   PAPI_MMAN
   
   
creation
   
   make,
   make_private,
   make_shared
   
   
feature -- creation
   
   make (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER;  a_base: POINTER; a_prot, a_flags: INTEGER) is
         -- raw interface to mmap
      do
         fd := a_fd
         offset := a_offset
         size := a_size
         base := posix_mmap (a_base, size, a_prot, a_flags, fd.value, offset)
         if base = MAP_FAILED then
            raise_posix_error
         end
      end
   
   make_private (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER ) is
         -- make a mapping where changes are private
         -- this function can fail on certain system (Linux for
         -- example) if a_offset is not a multiple of PAGE_SIZE
      do
         make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_PRIVATE)
      end
   
   make_shared (a_fd: POSIX_FILE_DESCRIPTOR; a_offset, a_size: INTEGER ) is
         -- make a mapping where changes are shared, i.e. the
         -- underlying object is also changed.
         -- this function can fail on certain system (Linux for
         -- example) if a_offset is not a multiple of PAGE_SIZE
      do
         make (a_fd, a_offset, a_size, default_pointer, PROT_READ + PROT_WRITE, MAP_SHARED)
      end
   
   
feature -- unmap
   
   close is
         -- remove the mapping
      do
         safe_call (posix_munmap (base, size))
         base := default_pointer
         size := 0
         offset := 0
      end
   
   
feature -- reading from the map
   
   peek_byte (index: INTEGER): INTEGER is
         -- consider memory an array of 8 bit values.
      require
         valid_index: index >= 0 and index < size
      do
         Result := posix_peek_byte (base, index)
      ensure
         possible_values: Result >= 0 and Result < 256         
      end
   
   
feature -- state

   base: POINTER
         -- base address
   
   offset: INTEGER
         -- offset from file
   
   size: INTEGER
         -- number of bytes mapping

   fd: POSIX_FILE_DESCRIPTOR
   
   
feature {NONE} -- POSIX c interface
   
   posix_peek_byte (p: POINTER; index: INTEGER): INTEGER is
         -- read byte at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end

   posix_peek_integer (p: POINTER; index: INTEGER): INTEGER is
         -- read integer at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end

   posix_poke_byte (p: POINTER; index: INTEGER; value: INTEGER) is
         -- read byte at position `index'
      require
         valid_memory: p /= default_pointer
      external "C"
      end
   

end -- class POSIX_MEMORY_MAP
