indexing

   description: "Class that covers the Standard C file system code. %
   %There was not much to cover..."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $";
   revision: "$Revision: 1.5 $"

class 

   STDC_FILE_SYSTEM

inherit

   STDC_BASE

   PORTABLE_PATH
      rename
         make_path as make
      end

   CAPI_STDIO
   
   
creation   
   
   make


feature -- rename files/directories, remove files/directories

   remove_file (a_path: STRING) is
         -- Removes a file from a directory
         -- it's not an error if this file does not exist
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         r: INTEGER
      do
         portable_path.make_expand (a_path)
         r := posix_remove (sh.string_to_pointer (portable_path))
         -- safe_call (posix_remove (sh.string_to_pointer (portable_path)))
      end
   
   rename_to (current_path, new_path: STRING) is
         -- Renames a file or directory
      require
         valid_current: current_path /= Void and then not current_path.is_empty
         valid_new: new_path /= Void and then not new_path.is_empty
      local
         c_current_path,
         c_new_path: POINTER
         expanded_new_path: STDC_PATH
      do
         portable_path.make_expand (current_path)
         c_current_path := sh.string_to_pointer (portable_path)
         create expanded_new_path.make (new_path)
         c_new_path := sh.string_to_pointer (expanded_new_path)
         safe_call (posix_rename (c_current_path, c_new_path))
      end


feature -- accessibility of files
   
   is_modifiable (a_path: STRING): BOOLEAN is
         -- tests if file is readable and writable by this program
         -- does this by attemting to open `a_path' file read/write
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         file: STDC_BINARY_FILE
         oops: BOOLEAN
      do
         if oops then
            Result := False
         else
            create file.open_read_write (a_path)
            file.close
            Result := True
         end
      rescue
         oops := True
         retry
      end
   
   is_readable (a_path: STRING): BOOLEAN is
         -- tests if file is readable by this program
         -- does this by attemting to open `a_path' file read-only
      require
         valid_path: a_path /= Void and then not a_path.is_empty
      local
         file: STDC_BINARY_FILE
         oops: BOOLEAN
      do
         if oops then
            Result := False
         else
            create file.open_read (a_path)
            file.close
            Result := True
         end
      rescue
         oops := True
         retry
      end


feature -- temporary names

   temporary_file_name, tmpnam: STRING is
         -- Generates a string that is a valid non-existing file name
      do
         Result := sh.pointer_to_string (posix_tmpnam (default_pointer))
      ensure
         valid_name: Result /= Void and then not Result.is_empty
      end


end -- class STDC_FILE_SYSTEM
